@@ -588,9 +588,11 @@ void JackTrip::completeConnection()
     for (auto& i : mProcessPluginsFromNetwork) {
         mAudioInterface->appendProcessPluginFromNetwork(i);
     }
+
     for (auto& i : mProcessPluginsToNetwork) {
         mAudioInterface->appendProcessPluginToNetwork(i);
     }
+
     mAudioInterface->initPlugins();   // mSampleRate known now, which plugins require
     mAudioInterface->startProcess();  // Tell JACK server we are ready for audio flow now
 
@@ -1050,13 +1052,17 @@ void JackTrip::stop(const QString& errorMessage)
     mHasShutdown = true;
     std::cout << "Stopping JackTrip..." << std::endl;
 
-    // Stop The Sender
-    mDataProtocolSender->stop();
-    mDataProtocolSender->wait();
+    if (mDataProtocolSender != nullptr) {
+        // Stop The Sender
+        mDataProtocolSender->stop();
+        mDataProtocolSender->wait();
+    }
 
-    // Stop The Receiver
-    mDataProtocolReceiver->stop();
-    mDataProtocolReceiver->wait();
+    if (mDataProtocolReceiver != nullptr) {
+        // Stop The Receiver
+        mDataProtocolReceiver->stop();
+        mDataProtocolReceiver->wait();
+    }
 
     // Stop the audio processes
     // mAudioInterface->stopProcess();
