@@ -50,7 +50,10 @@
 #include "jacktrip_globals.h"
 #ifdef _WIN32
 //#include <winsock.h>
+#include <stdio.h>
 #include <winsock2.h>  //cc need SD_SEND
+#pragma comment(lib, "ws2_32.lib")
+#define SIO_UDP_CONNRESET _WSAIOW(IOC_VENDOR, 12)
 #else
 #include <fcntl.h>
 #include <sys/socket.h>  // for POSIX Sockets
@@ -259,6 +262,14 @@ int UdpDataProtocol::bindSocket()
         local_addr.sin_port = htons(mBindPort);  // set local port
     }
 
+    // Prevent WSAECONNRESET errors that occur on Windows due to async UDP port setup
+#if defined(_WIN32)
+    BOOL bNewBehavior     = FALSE;
+    DWORD dwBytesReturned = 0;
+    WSAIoctl(sock_fd, SIO_UDP_CONNRESET, &bNewBehavior, sizeof bNewBehavior, NULL, 0,
+             &dwBytesReturned, NULL, NULL);
+#endif
+
     // Set socket to be reusable, this is platform dependent
     int one = 1;
 #if defined(_WIN32)
@@ -272,6 +283,31 @@ int UdpDataProtocol::bindSocket()
     ::setsockopt(sock_fd, SOL_SOCKET, SO_REUSEPORT, &one, sizeof(one));
 #endif
 
+#if defined(_WIN32)
+    // TODO: these don't seem to work on windows. we likely need to use qWAVE or qos2
+#elif defined(__APPLE__)
+    // set service type "Interactive Voice"
+    // TODO: this is supposed to be the right thing to do on OSX, but doesn't seem to do
+    // anything
+    const int val = NET_SERVICE_TYPE_VO;
+    ::setsockopt(sock_fd, SOL_SOCKET, SO_NET_SERVICE_TYPE, &val, sizeof(val));
+#else
+    // Set ToS to DSCP Expedited Forwarding (EF), recommended for Audio
+    // See RFC2474 https://datatracker.ietf.org/doc/html/rfc2474
+    // See also
+    // https://www.slashroot.in/understanding-differentiated-services-tos-field-internet-protocol-header
+    const char tos = 0xB8;  // 10111000
+    if (mIPv6) {
+        ::setsockopt(sock_fd, IPPROTO_IPV6, IPV6_TCLASS, &tos, sizeof(tos));
+    } else {
+        ::setsockopt(sock_fd, IPPROTO_IP, IP_TOS, &tos, sizeof(tos));
+    }
+
+    // Set 802.1q QoS priority
+    int priority = 6;
+    ::setsockopt(sock_fd, SOL_SOCKET, SO_PRIORITY, &priority, sizeof(priority));
+#endif
+
     // Bind the Socket
     if (mIPv6) {
         if ((::bind(sock_fd, (struct sockaddr*)&local_addr6, sizeof(local_addr6))) < 0) {
@@ -342,25 +378,25 @@ functions. DWORD n_bytes; WSABUF buffer; int error; buffer.len = n; buffer.buf =
 }
 
 //*******************************************************************************
-void UdpDataProtocol::getPeerAddressFromFirstPacket(QHostAddress& peerHostAddress,
-                                                    uint16_t& port)
-{
-    while (!datagramAvailable()) {
-        msleep(100);
-    }
-    char buf[1];
+// void UdpDataProtocol::getPeerAddressFromFirstPacket(QHostAddress& peerHostAddress,
+//                                                     uint16_t& port)
+// {
+//     while (!datagramAvailable()) {
+//         msleep(100);
+//     }
+//     char buf[1];
 
-    struct sockaddr_storage addr;
-    std::memset(&addr, 0, sizeof(addr));
-    socklen_t sa_len = sizeof(addr);
-    ::recvfrom(mSocket, buf, 1, 0, (struct sockaddr*)&addr, &sa_len);
-    peerHostAddress.setAddress((struct sockaddr*)&addr);
-    if (mIPv6) {
-        port = ((struct sockaddr_in6*)&addr)->sin6_port;
-    } else {
-        port = ((struct sockaddr_in*)&addr)->sin_port;
-    }
-}
+//     struct sockaddr_storage addr;
+//     std::memset(&addr, 0, sizeof(addr));
+//     socklen_t sa_len = sizeof(addr);
+//     ::recvfrom(mSocket, buf, 1, 0, (struct sockaddr*)&addr, &sa_len);
+//     peerHostAddress.setAddress((struct sockaddr*)&addr);
+//     if (mIPv6) {
+//         port = ((struct sockaddr_in6*)&addr)->sin6_port;
+//     } else {
+//         port = ((struct sockaddr_in*)&addr)->sin_port;
+//     }
+// }
 
 //*******************************************************************************
 void UdpDataProtocol::run()
@@ -647,7 +683,7 @@ void UdpDataProtocol::run()
                                         newer_seq_num);
             } else {
                 waitTime += 10;
-//                emit signalWaitingTooLong(waitTime);
+                emit signalWaitingTooLong(waitTime);
             }
         }
 #ifdef __linux__
