@@ -34,7 +34,10 @@
 #include <ctime>
 
 #include "about.h"
-#ifdef NO_JTVS
+#ifndef NO_VS
+#include "virtualstudio.h"
+#endif
+#ifdef PSI
 #include "ui_qjacktrip_novs.h"
 #else
 #include "ui_qjacktrip.h"
@@ -52,9 +55,9 @@
 #include "../Limiter.h"
 #include "../Reverb.h"
 
-QJackTrip::QJackTrip(int argc, QWidget* parent)
+QJackTrip::QJackTrip(int argc, bool suppressCommandlineWarning, QWidget* parent)
     : QMainWindow(parent)
-#ifdef NO_JTVS
+#ifdef PSI
     , m_ui(new Ui::QJackTrip)
 #else
     , m_ui(new Ui::QJackTripVS)
@@ -72,10 +75,6 @@ QJackTrip::QJackTrip(int argc, QWidget* parent)
 {
     m_ui->setupUi(this);
 
-    QCoreApplication::setOrganizationName(QStringLiteral("jacktrip"));
-    QCoreApplication::setOrganizationDomain(QStringLiteral("jacktrip.org"));
-    QCoreApplication::setApplicationName(QStringLiteral("JackTrip"));
-
     // Set up our debug window, and relay everything to our real cout.
     std::cout.rdbuf(m_debugDialog->getOutputStream()->rdbuf());
     std::cerr.rdbuf(m_debugDialog->getOutputStream(1)->rdbuf());
@@ -108,6 +107,10 @@ QJackTrip::QJackTrip(int argc, QWidget* parent)
         About about(this);
         about.exec();
     });
+#ifndef NO_VS
+    connect(m_ui->vsModeButton, &QPushButton::clicked, this,
+            &QJackTrip::virtualStudioMode);
+#endif
     connect(m_ui->autoPatchComboBox, QOverload<int>::of(&QComboBox::currentIndexChanged),
             this, [=]() {
                 if (m_ui->autoPatchComboBox->currentIndex() == CLIENTFOFI
@@ -234,6 +237,7 @@ QJackTrip::QJackTrip(int argc, QWidget* parent)
     m_ui->autoPatchGroupBox->setVisible(false);
     m_ui->requireAuthGroupBox->setVisible(false);
     m_ui->backendWarningLabel->setVisible(false);
+    m_ui->vsModeButton->setVisible(false);
 
 #ifdef RT_AUDIO
     connect(m_ui->backendComboBox, QOverload<int>::of(&QComboBox::currentIndexChanged),
@@ -277,7 +281,7 @@ QJackTrip::QJackTrip(int argc, QWidget* parent)
 
     // One of our arguments will always be --gui, so if that's the only one
     // then we don't need to show the warning message.
-    if ((!gVerboseFlag && m_argc > 2) || m_argc > 3) {
+    if (((!gVerboseFlag && m_argc > 2) || m_argc > 3) && !suppressCommandlineWarning) {
         QMessageBox msgBox;
         msgBox.setText(
             "The GUI version of JackTrip currently ignores any command line "
@@ -324,7 +328,7 @@ QJackTrip::QJackTrip(int argc, QWidget* parent)
             "JACK was not found. This means that only the RtAudio backend is available "
             "and that JackTrip cannot be run in hub server mode.");
 
-#ifdef NO_JTVS
+#ifdef PSI
         QSettings settings;
         settings.beginGroup(QStringLiteral("Audio"));
         if (!settings.value(QStringLiteral("HideJackWarning"), false).toBool()) {
@@ -360,7 +364,7 @@ QJackTrip::QJackTrip(int argc, QWidget* parent)
             settings.setValue(QStringLiteral("UsingFallback"), false);
         }
         settings.endGroup();
-#endif  // NO_JTVS
+#endif  // PSI
 #else   // RT_AUDIO
         QMessageBox msgBox;
         msgBox.setText(
@@ -415,6 +419,37 @@ void QJackTrip::resizeEvent(QResizeEvent* event)
     m_ui->authDisclaimerLabel->setMinimumHeight(rect.height());
 }
 
+void QJackTrip::showEvent(QShowEvent* event)
+{
+    // We need to wait to load geometry until here rather than with our other settings.
+    // If we don't, the window geometry will be improperly set on macOS whenever the
+    // VirtualStudio window is shown first.
+    QMainWindow::showEvent(event);
+    if (m_firstShow) {
+        QSettings settings;
+        settings.beginGroup(QStringLiteral("Window"));
+        QByteArray geometry = settings.value(QStringLiteral("Geometry")).toByteArray();
+        if (geometry.size() > 0) {
+            restoreGeometry(geometry);
+        } else {
+            // Because of hidden elements in our dialog window, it's vertical size in the
+            // creator is getting rediculous. Set it to something sensible by default if
+            // this is our first load.
+            this->resize(QSize(this->size().height(), 600));
+        }
+        settings.endGroup();
+        m_firstShow = false;
+    }
+}
+
+#ifndef NO_VS
+void QJackTrip::setVs(QSharedPointer<VirtualStudio> vs)
+{
+    m_vs = vs;
+    m_ui->vsModeButton->setVisible(!m_vs.isNull());
+}
+#endif
+
 void QJackTrip::processFinished()
 {
     if (!m_jackTripRunning) {
@@ -444,7 +479,7 @@ void QJackTrip::processError(const QString& errorMessage)
 {
     QMessageBox msgBox;
     if (errorMessage == QLatin1String("Peer Stopped")) {
-        // Report the other end quitting as a regular occurance rather than an error.
+        // Report the other end quitting as a regular occurrence rather than an error.
         msgBox.setText(errorMessage);
         msgBox.setWindowTitle(QStringLiteral("Disconnected"));
     } else {
@@ -670,6 +705,7 @@ void QJackTrip::resetOptions()
     m_ui->realTimeCheckBox->setChecked(true);
     m_ui->ioStatsCheckBox->setChecked(false);
     m_ui->ioStatsSpinBox->setValue(1);
+    m_ui->verboseCheckBox->setChecked(false);
 
     saveSettings();
 }
@@ -943,6 +979,14 @@ void QJackTrip::exit()
     }
 }
 
+#ifndef NO_VS
+void QJackTrip::virtualStudioMode()
+{
+    this->hide();
+    m_vs->show();
+}
+#endif
+
 int QJackTrip::findTab(const QString& tabName)
 {
     for (int i = 0; i < m_ui->optionsTabWidget->count(); i++) {
@@ -1177,18 +1221,6 @@ void QJackTrip::loadSettings()
     m_ui->outClientsSpinBox->setValue(
         settings.value(QStringLiteral("Clients"), 1).toInt());
     settings.endGroup();
-
-    settings.beginGroup(QStringLiteral("Window"));
-    QByteArray geometry = settings.value(QStringLiteral("Geometry")).toByteArray();
-    if (geometry.size() > 0) {
-        restoreGeometry(geometry);
-    } else {
-        // Because of hidden elements in our dialog window, it's vertical size in the
-        // creator is getting rediculous. Set it to something sensible by default if this
-        // is our first load.
-        this->resize(QSize(this->size().width(), 600));
-    }
-    settings.endGroup();
 }
 
 void QJackTrip::saveSettings()
